﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Reflection;

namespace howto_reflection_list_fields
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Add some fields.
        private int MyPrivateField = 1;
        public int MyPublicField = 2;
        public static int MyPublicStaticField = 3;
        public static int MyProtectedField = 4;
        public const int MyConstField = 5;
        public int[] MyArray = { 1, 2, 3, 4, 5 };
        public int[] MyEmptyArray = { };
        public int[] MyNullArray;

        private void Form1_Load(object sender, EventArgs e)
        {
            // Make column headers.
            lvwProperties.Columns.Clear();
            lvwProperties.Columns.Add("Field", 10,
                HorizontalAlignment.Left);
            lvwProperties.Columns.Add("Type", 10,
                HorizontalAlignment.Left);
            lvwProperties.Columns.Add("Attributes", 10,
                HorizontalAlignment.Left);
            lvwProperties.Columns.Add("Visibility", 10,
                HorizontalAlignment.Left);
            lvwProperties.Columns.Add("Value", 10,
                HorizontalAlignment.Left);

            // List the fields.
            // Use the class you want to study instead of Form1.
            object property_value;
            FieldInfo[] field_infos = typeof(Form1).GetFields(
                BindingFlags.FlattenHierarchy |
                BindingFlags.Instance |
                BindingFlags.NonPublic |
                BindingFlags.Public |
                BindingFlags.Static);
            foreach (FieldInfo info in field_infos)
            {
                string name = info.Name;
                string attributes = info.FieldType.Attributes.ToString();

                string visibility = "";
                if (info.IsAssembly) visibility += " assembly";
                if (info.IsFamily) visibility += " family";
                if (info.IsFamilyAndAssembly) visibility += " family AND assembly";
                if (info.IsFamilyOrAssembly) visibility += " family OR assembly";
                if (info.IsInitOnly) visibility += " init";
                if (info.IsLiteral) visibility += " literal";
                if (info.IsPrivate) visibility += " private";
                if (info.IsPublic) visibility += " public";
                if (info.IsStatic) visibility += " static";
                if (visibility.Length > 0) visibility = visibility.Substring(1);

                string value = "";

                // See if it's an array.
                if (!info.FieldType.IsArray)
                {
                    // It's not an array.
                    property_value = info.GetValue(this);
                    if (property_value == null)
                        value = "<null>";
                    else
                        value = property_value.ToString();
                }
                else
                {
                    // It is an array.
                    name += "[]";
                    // If it's an array of integers, get the values.
                    if (info.FieldType.Name == "Int32[]")
                    {
                        Int32[] values = (Int32[])info.GetValue(this);
                        if (values == null)
                        {
                            value = "<null>";
                        }
                        else
                        {
                            foreach (Int32 val in values)
                                value += ", " + val.ToString();
                            if (value.Length > 0) value = value.Substring(2);
                            value = "[" + value + "]";
                        }
                    }
                    else
                    {
                        value = "<array>";
                    }
                }

                ListViewMakeRow(lvwProperties, name,
                    info.FieldType.ToString(),
                    attributes, visibility, value);
            }

            // Size the columns to fit the data.
            for (int i = 0; i < lvwProperties.Columns.Count; i++)
                lvwProperties.Columns[i].Width = 100;

            // Size the form.
            int new_wid = 30;
            for (int i = 0; i < lvwProperties.Columns.Count; i++)
                new_wid += lvwProperties.Columns[i].Width;
            this.Size = new Size(new_wid, this.Size.Height);
        }

        // Make a ListView row.
        private void ListViewMakeRow(ListView lvw, String item_title, params string[] subitem_titles)
        {
            // Make the item.
            ListViewItem new_item = lvw.Items.Add(item_title);

            // Make the sub-items.
            for (int i = subitem_titles.GetLowerBound(0); i <= subitem_titles.GetUpperBound(0); i++)
            {
                new_item.SubItems.Add(subitem_titles[i]);
            }
        }
    }
}
